package cq.flink.kafka.mqtt.funcs;


import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.java.tuple.Tuple2;
import org.apache.flink.util.Collector;

@Slf4j
public class LogFilterMapFunction implements FlatMapFunction<Tuple2<String, String>, Tuple2<String, String>> {
    
    private final ObjectMapper om = new ObjectMapper();
    private final static String TOPIC_FILTER_PREFIX = "cumtcsb617";
    private final boolean iotFilterEnable = false;

    @Override
    public void flatMap(Tuple2<String, String> value, Collector<Tuple2<String, String>> out) throws Exception {
        // 过滤空数据
        if (value.f1 == null) {
            log.warn("TOPIC-[{}] 过滤空数据...", value.f0);
            return;
        }
        value.f1 = value.f1.trim();
        if (value.f1.length() > 2) {
            if (isValidJsonString(value.f1)) {
                try {
                    // 认为是 JSON 数据
                    JsonNode root = om.readTree(value.f1);
                    // 过滤无效数据
                    if (iotFilterEnable && value.f0.startsWith(TOPIC_FILTER_PREFIX)) {
                        if (root.get("type") != null || root.get("messageType") != null) {
                            // 确认为无效数据
                            return;
                        }
                    }
                    // 过滤掉 dataByteArray 字段
                    if (root.get("dataByteArray") != null){
                        ObjectNode objectNode = (ObjectNode) root;
                        objectNode.remove("dataByteArray");
                        value.f1 = om.writeValueAsString(objectNode);
                    }
                    log.info("TOPIC-[{}] 转发 JSON 数据 -> {}", value.f0, value.f1.subSequence(0, Math.min(value.f1.length(), 1 << 5)));
                    out.collect(value);
                } catch (Exception e) {
                    log.error("TOPIC-[{}] JSON 数据解析失败 -> [{}]", value.f0, value.f1, e);   
                }
                return;
            }
        }
        // 处理非 JSON 数据 直接转发
        log.info("TOPIC-[{}] 直接转发非 JSON 数据 -> {}", value.f0, value.f1.subSequence(0, Math.min(value.f1.length(), 1 << 5)));
        out.collect(value);
        
    }


    private static boolean isValidJsonString(String json) {
        char firstChar = json.charAt(0);
        char lastChar = json.charAt(json.length() - 1);
        if ( (firstChar == '{') && (lastChar == '}') ) {
            return true;
        } else return (firstChar == '[') && (lastChar == ']');
    }

}