package cq.flink.kafka.mqtt.sink;

import io.vertx.core.Vertx;
import io.vertx.core.VertxOptions;
import io.vertx.mqtt.MqttClient;
import io.vertx.mqtt.MqttClientOptions;

import java.io.Serializable;
import java.util.UUID;

// 客户端管理
public class MqttClientManager implements Serializable {

  private volatile static Vertx vertx = null;
  public synchronized static Vertx getVertx() {
    if (vertx == null) {
      VertxOptions ops = new VertxOptions();
      ops.setWorkerPoolSize(1 << 3);
      ops.setEventLoopPoolSize(1);
      vertx = Vertx.vertx(ops);
    }
    return  vertx;
  }
  
  
  public static MqttClient createClient(int maxInflightQueue, int maxMessageSize) {
    MqttClientOptions ops = new MqttClientOptions();
    ops.setAutoKeepAlive(true);
    ops.setReconnectAttempts(10);
    ops.setReconnectInterval(1000);
    ops.setClientId("flink-vertx_mqtt-sink_" + UUID.randomUUID().toString().replaceAll("-", ""));
    ops.setConnectTimeout(10000);
    if (maxMessageSize > 0) {
      ops.setMaxMessageSize(maxMessageSize);
    }
    ops.setMaxInflightQueue(maxInflightQueue);
    return MqttClient.create(getVertx(), ops);
  }

}