package cq.flink.kafka.mqtt.source;

import cq.flink.kafka.mqtt.cfg.ApplicationCfg;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.java.tuple.Tuple2;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.KafkaSourceBuilder;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;

import java.util.UUID;
import java.util.regex.Pattern;

@Slf4j
public class KafkaSourceCreator {

    public static KafkaSource<Tuple2<String, String>> createSourceOfString(ApplicationCfg cfg) {
        KafkaSourceBuilder<Tuple2<String, String>> builder = KafkaSource.<Tuple2<String, String>>builder()
                .setBootstrapServers(cfg.getBootstrapServers());
        if (cfg.getTopic() != null) {
            if (cfg.getTopic().getNames() != null && cfg.getTopic().getNames().length > 0) {
                builder.setTopics(cfg.getTopic().getNames());
            } else if (cfg.getTopic().getPattern() != null) {
                builder.setTopicPattern(Pattern.compile(cfg.getTopic().getPattern()));
            }
            if (cfg.getTopic().getDiscoveryInterval() > 0) {
                builder.setProperty("partition.discovery.interval.ms", String.valueOf(cfg.getTopic().getDiscoveryInterval()));
            }
        }
        if ("earliest".equalsIgnoreCase(cfg.getStartingOffset())) {
            builder.setStartingOffsets(OffsetsInitializer.earliest());
        } else {
            builder.setStartingOffsets(OffsetsInitializer.latest());
        } 
        return builder.setGroupId(cfg.getGroupId() == null ? "flink-Kafka-mqtt-" + UUID.randomUUID() : cfg.getGroupId())
                .setDeserializer(new WithTopicDeserialization())
                .build();
    }

}